(function ($) {
  $.fn.changeElementType = function (newType) {
    var attrs = {};

    $.each(this[0].attributes, function (idx, attr) {
      attrs[attr.nodeName] = attr.nodeValue;
    });

    this.replaceWith(function () {
      return $("<" + newType + "/>", attrs).append($(this).contents());
    });
  };
})(jQuery);

$('#manuscript').append('<div id="text_resize" style="color: transparent" aria-hidden="true">X</div>');

function handleTextResize(event, args) {
  /*
  console.log('caught text resize event');
  console.log(event);
  console.log(args);
  */
  
  resizeIframe();
}

function resizeIframe() {
  var $iframeDiv = $('[data-figure_iframe="true"]');

  if ($iframeDiv.length < 1) {
    return;
  }

  var $iframe = $iframeDiv.children('iframe');

  if ($iframe.length < 1) {
    return;
  }

  var iframeContentHeight = calculateFigureIframeHeight($iframe);

  if (iframeContentHeight > 0) {

    //console.log('resizeIframe: setting iframe height to ' + iframeContentHeight);

    $iframeDiv.css('height', iframeContentHeight + 'px');
    $iframe.attr('height', iframeContentHeight);
  }
}

function calculateFigureIframeHeight($iframe) {
  return $iframe.contents().find('#data-visualization-activities').height();
}

function initTextResize() {
  //console.log('initTextResize');
  TextResizeDetector.addEventListener(handleTextResize, null);
}



//id of element to check for and insert control
TextResizeDetector.TARGET_ELEMENT_ID = 'text_resize';
//function to call once TextResizeDetector has init'd
TextResizeDetector.USER_INIT_FUNC = initTextResize;

$('div[data-type="question"]').each(function () {
  $(this).hide();
});

$('div[data-block_type="paragraph_left"]').each(function () {
  $(this).hide();
});

var g2dContainer,
  currentG2DContainerHeight,
  descriptionParams = {
    flag: false,
    height: 315
  };
var iframeContainer,
  currentIfameContanerHeight;

function alignVisibleChoicesCells() {
  var visibleChoices = $('[data-type="question"]:not([style="display: none;"]) .query_mc_choice_table');
  var maxVisibleChoiceHeight = Math.max.apply(null, visibleChoices.map(function (index, el) {
    return $(el).outerHeight();
  }));
  visibleChoices.css('height', maxVisibleChoiceHeight + 'px');
}

function alignSubmitBtnByLastChoice() {
  var visibleQuestion = $('[data-type="question"]:not([style="display: none;"])');
  var checkboxTable = visibleQuestion.find('div[data-type="table"] table');
  var currentlastVisibleChoice = visibleQuestion.find('.query_mc_choice_table').eq(3);
  var newBattonsNavWidth;
  if (currentlastVisibleChoice.length) {
    newBattonsNavWidth = (currentlastVisibleChoice.position().left + currentlastVisibleChoice.outerWidth() + 30); // 30 - choiceMargin
    $('.buttonsNav').css('width', newBattonsNavWidth + 'px');
  }
  else if (checkboxTable.length) {
    newBattonsNavWidth = (checkboxTable.position().left + checkboxTable.outerWidth());
    $('.buttonsNav').css('width', newBattonsNavWidth + 'px');
  }
}

function customizeVisibleQuestionRadioButtons() {
  var labels = $('[data-type="question"]:not([style="display: none;"]) label'),
    inputContaners = $('[data-type="question"]:not([style="display: none;"]) td div');
  for (var i = 0; i < labels.length; i++) {
    labels.eq(i).appendTo(inputContaners.eq(i));
  }
}

var timerFigureonLoad = setInterval(function () {
  if ($('div[data-type="figure"] iframe').length) {
    clearInterval(timerFigureonLoad);

    var timerFigureonContentLoad = setInterval(function () {
      var iframeBody = $('div[data-type="figure"] iframe')[0].contentWindow.document.body;
      if ($(iframeBody).find('#btnLongDescription').length) {
        clearInterval(timerFigureonContentLoad);

        var iframeDiv = $('div[data-type="figure"] iframe');
        var $iframeBody = $(iframeDiv[0].contentWindow.document.body);
        var title = $('h1');
        $(iframeDiv).attr('aria-label', title.html());
        $(iframeDiv).attr('title', title.html());


        //fix issue with focusable iframe's body element for IE and Firefox
        var G2DBoxElement = iframeBody.querySelector('div.G2Dbox');
        var navBackButton = $(".button_back");

        G2DBoxElement.addEventListener('keydown', function (event) {
          if (event.shiftKey && event.keyCode == 9) {
            event.preventDefault();
            setTimeout(function () {
              navBackButton.focus();
            }, 100);
          }
        });

        navBackButton.bind('keydown', function (event) {
          if (!event.shiftKey && event.keyCode == 9) {
            event.preventDefault();
            setTimeout(function () {
              G2DBoxElement.focus();
            }, 100);
          }
        });

        //fix issue with double reading of fieldset legend on Mac VoiceOver;
        //legend have attribute aria-hidden. We update aria-live element when query get focus
        if (platform.mac) {
          var $ariaLiveElement = $iframeBody.find('#activeDescription');
          var $descriptionButton = $iframeBody.find('#btnLongDescription');
          var $submitButtons = $('.question_submit_button');

          $descriptionButton.bind('keydown', function (event) {
            if (event.keyCode == 9) {
              setTimeout(updateAriaLive, 100);
            }
          });

          $submitButtons.each(function () {
            $(this).bind('keydown', function (event) {
              if (event.shiftKey && event.keyCode == 9) {
                setTimeout(updateAriaLive, 100);
              }
            });
          });

          $('fieldset').each(function () {
            $(this).bind('focusout', function (e) {
              $ariaLiveElement.html(' ');
            });

          });

          function updateAriaLive() {
            var $activeQuestionLegend = $('legend:visible');
            var radio = document.activeElement;
            var activeRadioDescription = $(radio).next().text();

            $ariaLiveElement.text($activeQuestionLegend.text() + activeRadioDescription + (radio.checked ? '. Selected' : '') + ', radio button');
          }
        }

        $('title')[0].innerHTML = title[0].innerHTML;

        //$(iframeDiv[0]).attr('role', 'none');

        $("strong").replaceWith(function (index, oldHTML) {
          return $("<h2>").html(oldHTML);
        });

        $('div.section_title').attr('role', 'banner');
        //$('div[data-type="section"]').attr('role', 'main');
        $('div.buttonsNav').attr('role', 'navigation');

        $('button,input[type="button"]').each(function () {
          $(this).attr('role', 'button');
        });

        $('div[data-type="question"]').each(function () {
          var index = parseInt($(this).attr('data-question_index'), 10);

          var question = $('div[data-question_index="' + index + '"]');

          // brb: if isSubmit is already set to true (because the
          // question has been previously answered and set to true
          // in restore_saved_answers) then do not set to false here

          var is_submit = question.attr('isSubmit');

          if (is_submit != "true") {
            question.attr('isSubmit', false);
          }

          var length = $('div[data-type="question"]').length;

          var current = question[0];
          while ($(current).prev().attr('data-block_type') == "paragraph_left" || $(current).prev().children().attr('data-block_type') == "paragraph_left") {
            if ($(current).prev().attr('data-block_type') == "paragraph_left") {
              $(current).prev().attr('helper_for_data_question_index', index);
              current = $(current).prev();
            }
            else if ($(current).prev()[0].childNodes[0].getAttribute('data-block_type') == "paragraph_left") {
              $($(current).prev()[0].childNodes[0]).attr('helper_for_data_question_index', index);
              current = $(current).prev()[0].childNodes[0];
            }
            if ($(current).prev()[0] == undefined) {
              break;
            }
          }

          var navDiv = $('<div class="buttonsNav" style="display: none;" view_question="' + index + '"></div>');
          var questionNumberDiv = $('<div class="questionNumber"><span class="questionCounter">' + index + '</span> / ' + length + '</div>');
          var nextBtnDiv = $('<button class="buttonNext questionNavButton" style="display: none;">Next</button>');
          var backBtnDiv = $('<button class="buttonBack questionNavButton">Back</button>');


          if (index === 1) {
            navDiv.append(questionNumberDiv).append(nextBtnDiv);
          }
          else if (length === index) {
            navDiv.append(backBtnDiv).append(questionNumberDiv);
          }
          else {
            navDiv.append(backBtnDiv).append(questionNumberDiv).append(nextBtnDiv);
          }
          question.parent().append(navDiv);

          var buttonBack = $('.buttonsNav[view_question="' + index + '"] .buttonBack');
          var buttonNext = $('.buttonsNav[view_question="' + index + '"] .buttonNext');

          if (buttonBack.length) {
            buttonBack.attr({
              'aria-label': 'Go back to question ' + (index - 1),
              'view_question': index - 1
            });

            buttonBack.bind('click', function () {
              question.hide();
              $(this).parent().hide();

              var curBtnPanel = $('.buttonsNav[view_question="' + (index - 1) + '"]');
              var curQuestion = $('div[data-question_index="' + (index - 1) + '"]');

              curBtnPanel.show();
              curQuestion.show();

              $('div[helper_for_data_question_index="' + index + '"]').hide();
              $('div[helper_for_data_question_index="' + (index - 1) + '"]').show();
              curBtnPanel.find('.buttonNext').focus()
            });
          }

          if (buttonNext.length) {
            buttonNext.attr({
              //'aria-label': 'Next question number ' + (index + 1),
              'aria-label': 'Go to question ' + (index + 1),
              'view_question': index + 1
            });

            buttonNext.bind('click', function () {

              var isSubmit = $('div[data-question_index="' + index + '"]').attr('isSubmit');

              if (isSubmit == 'false') {
                Standard_Dialog.alert('You must answer the question before moving on to the next slide.', {
                  'title': 'Message',
                  'buttons': 'OK',
                  'from': buttonNext
                });
                return;
              }

              question.hide();
              $(this).parent().hide();

              var curBtnPanel = $('.buttonsNav[view_question="' + (index + 1) + '"]');
              var curQuestion = $('div[data-question_index="' + (index + 1) + '"]');

              curBtnPanel.show();

              var attrViewQuestion = $('div[data-question_index="' + $(this).attr('view_question') + '"]');
              attrViewQuestion.show();

              $('div[helper_for_data_question_index="' + index + '"]').hide();
              $('div[helper_for_data_question_index="' + (index + 1) + '"]').show();
              if (curQuestion[0].getAttribute('issubmit') === 'true') {
                //console.log('setting focus on Next button');
                curBtnPanel.find('.buttonNext').focus();
              }
              else {
                var $set_focus = attrViewQuestion.find('[data-block_type="set_focus"]');
                //console.log('$set_focus = ', $set_focus);
                if ($set_focus.length > 0) {
                  $set_focus.focus();
                }
                else {
                  $(attrViewQuestion.find('input')[0]).focus();
                }
              }

              customizeVisibleQuestionRadioButtons();
              alignVisibleChoicesCells();
              alignSubmitBtnByLastChoice();

              //fix issue with double reading of fieldset legend on Mac VoiceOver;
              //legend have attribute aria-hidden. We update aria-live element when query get focus
              if (platform.mac) {
                var iframeDiv = $('div[data-type="figure"] iframe');
                var $iframeBody = $(iframeDiv[0].contentWindow.document.body);
                var $ariaLiveElement = $iframeBody.find('#activeDescription');
                var $activeQuestionLegend = $('legend:visible');
                var activeRadioDescription = attrViewQuestion.find('input').eq(0).next().text();
                $ariaLiveElement.text($activeQuestionLegend.text() + ' ' + activeRadioDescription + ', radio button');
              }
            });
          }
        });

        $('.question_submit_button').each(function () {
          $(this).bind('click', function () {

            safe_log('click handler for submit button');


            var index = $($(this).closest('div[data-type="question"]')).attr('data-question_index');

            // only proceed if the question has been answered
            if (!player.activity.questions[index].queries[0].isAnswered()) {
              return;
            }

            // if force_at_least_one_cb_answer is true then only proceed if at least one CB box has been checked
            if (player.activity.questions[index].md && player.activity.questions[index].md.force_at_least_one_cb_answer === 'true') {
              if (!player.hasAtLeastOneCBChecked(index)) {
                return;
              }
            }

            $('div[data-question_index="' + index + '"]').attr('isSubmit', true);

            var buttonNavPanel = $('.buttonsNav[view_question="' + index + '"] .buttonNext');
            if (buttonNavPanel.length) {
              buttonNavPanel.show();
              //console.log('setting focus on Next button (2)');
              // allow feedback announcement to come before button announcement
              // when using a screen reader
              setTimeout(function () {
                buttonNavPanel.focus();
              }, 500);
              //buttonNavPanel.focus();
            } else {
            }

            $(this).hide();
            customizeVisibleQuestionRadioButtons();
            alignVisibleChoicesCells();

            //last question feedback is readed by screen reader because of focus on the DOM element (IE only) .
            // Don't update aria live element to prevent double reading
            var isIE = window.navigator.msPointerEnabled;
            if (index == 10 && isIE) return;

            var descNode = $iframeBody.find('#activeDescription');

            var firstPart = '';
            var secondPart = '';
            var questionFeedback = $('#question_feedback_' + index);
            var questionAction = $('#question_action_div_' + index);

            if (questionFeedback[0].childNodes[1]) {
              firstPart = questionFeedback[0].childNodes[1].innerHTML.replace(/<br>/g, '')
            }
            if (questionAction[0].childNodes[1]) {
              secondPart = questionAction[0].childNodes[1].innerHTML;
            }

            var desc = firstPart + ' ' + secondPart;

            var s = player.get_current_section();

            descNode.html(' ');

            var iOS = !!navigator.platform && /iPad|iPhone|iPod/.test(navigator.platform);
            var ua = navigator.userAgent.toLowerCase();
            var isAndroid = ua.indexOf("android") > -1; //&& ua.indexOf("mobile");


            if (iOS || isAndroid) {
              descNode.html(s.all_questions_answered ? desc + ' ' + descNode.html() + ' Next button' : desc + ' Next button');
            }
            else {
              descNode.html(s.all_questions_answered ? desc + ' ' + descNode.html() : desc);
            }

          });
        });

        var index = 1;

        var question = $('div[data-question_index="' + index + '"]');

        if (question.attr('wasopened') === undefined) {
          question.attr('wasopened', 1);
          $('.buttonsNav[view_question="' + index + '"]').show();
          // var optionsString = '';
          var options = $('div[data-question_index="' + (index) + '"]');
          var optionsLength = options[0].querySelectorAll('label').length;
          // for (var i = 0; i < optionsLength; ++i) {
          //   optionsString += options[0].querySelectorAll('tr')[i].childNodes[1].innerHTML + ' ' +
          //     options[0].querySelectorAll('label')[i].innerHTML + ' ';
          // }
        }
        question.show();

        $('div[helper_for_data_question_index="' + (index) + '"]').show();

        var descriptionText = $iframeBody.find('.longDescriptionText'),
          descriptionArrow = $iframeBody.find('.descArrow');

        //There is bug in FF. FF doesn't see height when iframe is hidden
        var descriptionTextTrueHegiht = descriptionText.height() + 32; // 32 = ( padding = 10 + border = 2 + margin = 20 )


        $iframeBody.find('#btnLongDescription')[0].setAttribute('aria-label', 'Collapsed Image Description');

        $iframeBody.find('#btnLongDescription')
          .bind('click', function () {

            this.disabled = true; //disable button to prevent call of handler while text slide

            var desc = $iframeBody.find('#graph-aria-longDescription');
            var descNode = $iframeBody.find('#activeDescription');
            var longDescriptionText = $iframeBody.find('.longDescriptionText');

            descriptionTextTrueHegiht = descriptionText.height() + 32; //for FF. FF doesn't see height when iframe is hidden


            descNode[0].innerHTML = ' ';
            descNode[0].innerHTML = desc[0].innerHTML;
            longDescriptionText[0].innerHTML = ' ';
            longDescriptionText[0].innerHTML = desc[0].innerHTML;

            if (descriptionText.css('display') === 'none') {
              descriptionText.slideDown(1);
              descriptionArrow.css('transform', 'rotate(90deg)');
              if (g2dContainer.css('flex-direction') === 'column') {
                descriptionParams.flag = true;
                descriptionParams.height = descriptionTextTrueHegiht;
                this.setAttribute('aria-label', ' Expanded Image Description');
              }
            }
            else {
              descNode[0].innerHTML = ' ';
              descriptionText.slideUp(1);
              descriptionArrow.css('transform', 'rotate(0deg)');
              if (g2dContainer.css('flex-direction') === 'column') {
                descriptionParams.flag = true;
                descriptionParams.height = -descriptionTextTrueHegiht;
              }
              this.setAttribute('aria-label', 'Collapsed Image Description');
            }
            $(window).resize();


            setTimeout(function () {
              handleTextResize();
            }, 100);


            var ctx = this;
            setTimeout(function () {
              ctx.disabled = false;
            }, 700)

          });

        $iframeBody.find('.selectpicker')
          .bind('click', function () {
            setTimeout(function () { // setTimeout - fix for android, when additional elements appear
              $(window).resize();
            }, 100);
          });

        function resizeIframeContainer() {
          //console.log('resizeIframeContainer');
          var g2dConteinerHeight = g2dContainer.height(); //.commonG2D-container

          /* brb: We don't want to do this resize any more because it doesn't
             work correctly and we are doing it a better way now. */
          /*
          if (descriptionParams.flag) { // if narrow screen. Description and controls are situated under graph
            setTimeout(function () {
              currentIfameContanerHeight = currentIfameContanerHeight + descriptionParams.height;

              iframeContainer.css('height', currentIfameContanerHeight + 'px'); //iframeContainer wrap iframe tag
              descriptionParams.flag = false;
            }, 0);

            setTimeout(function () {
              currentG2DContainerHeight = g2dContainer.height();
            }, 600)
          }
          else { //if wide screen. Description and controls are situated in a row
            if (g2dConteinerHeight != currentG2DContainerHeight) {
              if (currentIfameContanerHeight) {
                currentIfameContanerHeight = currentIfameContanerHeight + (g2dConteinerHeight - currentG2DContainerHeight);
                iframeContainer.css('height', currentIfameContanerHeight + 'px');
              }
              currentG2DContainerHeight = g2dConteinerHeight;
            }
          }
          */

          // align choices cells after orientation changing on big tablets
          $('[data-type="question"]:not([style="display: none;"]) .query_mc_choice_table').css('height', 'auto');
          alignVisibleChoicesCells();
          alignSubmitBtnByLastChoice();
        } // end resizeIframeContainer

        $(window).resize(resizeIframeContainer);
      }
    }, 100);
  }
}, 100);

var Activity_datavis = Activity_manuscript_type.extend({

  // brb: We need to redefine this here so we can add the 'isSubmit'
  // attribute onto restored answers
  restore_saved_answers: function () {
    // skip page queries (questions[0])
    for (var i = 1; i < this.questions.length; ++i) {
      var question = this.questions[i];

      // prepare for question_report_text questions
      var query_responses;
      if (question.md_set("question_report_text")) {
        query_responses = "" + Get_ARGA_QuestionData(this.get_question_number(question))
        query_responses = query_responses.split(this.query_responses_separator);
      }

      var restored_all_answers = true;
      var all_answers_correct = true;
      for (var j = 0; j < question.queries.length; ++j) {
        var query = question.queries[j];

        var saved_answer, saved_grade;

        // get saved answer and grade out of the questionData for question_report_text questions
        if (question.md_set("question_report_text")) {
          if (query_responses[j] != null) {
            var arr = query_responses[j].split(this.query_responses_grade_separator);
            saved_answer = arr[0];
            saved_grade = arr[1];
          } else {
            saved_grade = saved_answer = "";
          }

          // or out of learnerResponse / questionData for single-query questions
        } else {
          var ARGA_question_number = this.get_query_number(question, j);
          saved_answer = Get_ARGA_LearnerResponse(ARGA_question_number);
          // we stored the original grade in questionData
          saved_grade = Get_ARGA_QuestionData(ARGA_question_number)
        }

        // if we found a grade...
        if (saved_grade !== "" && saved_grade >= 0) {
          // restore the answer
          query.restoreAnswerFromARGA(saved_answer);
          query.setGrade(saved_grade);

          // then re-render the query in review mode
          query.jq.html(query.getHTML("review"));

          if (query.isCorrect() == false) {
            all_answers_correct = false;
          }

        } else {
          restored_all_answers = false;
        }
      }

      if (restored_all_answers) {
        // update the submit div for the question
        this.update_question_action_div(i);

        // Show appropriate feedback. If the question responses were submitted, 
        // the student must have finished the question, so we don't have to worry
        // about hints.
        this.show_question_feedback(question, all_answers_correct, false);

        // if the item is in a sequence, show the next question
        this.show_next_question(question);

        // NEED TO CHECK TO MAKE SURE THIS IS WORKING PROPERLY
        // If the question is in a sequence and the sequence is complete, call show_closing_material
        var qs = question.question_sequence;
        if (qs != null && qs.sequence_complete == "partial") {
          this.show_closing_material(qs.index);
        }

        // update section status
        player.update_section_status();

        $("[data-question_index=" + i + "]").attr("isSubmit", true);
      }
    }
  },



  submit_question: function (question_index) {

    $('.question_action_div_error').remove();

    // don't allow question to be submitted unless answer has been selected
    var q_idx = question_index;
    if (typeof q_idx == "object") {
      // extract the question_index from the object
      q_idx = question_index.data.question_index;
    }

    var query_type = player.activity.questions[q_idx].queries[0].getQueryTypeForARGA();
    //console.log('query type = ' + query_type);

    // if question has force_at_least_one_cb_answer set to true then make sure
    // at least one CB box has been checked
    if (player.activity.questions[q_idx].md && player.activity.questions[q_idx].md.force_at_least_one_cb_answer === "true") {
      //console.log('force_at_least_one_cb_answer is in play');
      if (!player.hasAtLeastOneCBChecked(q_idx)) {
        //console.log('no checkbox has been checked');
        $('#question_action_div_' + q_idx).before(
          '<div class="question_action_div_error">You must select at least one answer before submitting.</div>'
        );
        player.shadow_speech.talk('You must select at least one answer before submitting.');
        return;
      }
    }

    // if this is a MC question make sure the user has selected an answer
    if (query_type === 'multiple choice') {
      player.activity.questions[q_idx].queries[0].setError();

      safe_log('submit_question: question index = ' + q_idx);
      safe_log(player.activity.questions[q_idx]);

      if (!player.activity.questions[q_idx].queries[0].isAnswered()) {
        player.activity.questions[q_idx].queries[0].setError('Please select an answer before submitting.');
        player.shadow_speech.talk('Please select an answer before submitting.');
        return;
      }
    }

    this._super(question_index);

    var s = player.get_current_section();

    // if all questions are answered and all sequences are complete
    // (these are set by _super)
    if (s.all_questions_answered && s.all_question_sequences_completed) {
      // if we're not on the final step
      if (s.index < player.sections.length - 1) {
        // otherwise we're done!
      } else {
        //delay dialog alert that contain quiz's results to allow screenreader read last question's feedback
        setTimeout(function () {
          player.show_completion_message();
        }, 1500);
        // player.show_completion_message();
      }
    }
  },
  show_question_feedback: function (question, is_correct, try_again) {
    // remove prior feedback if there
    $("#question_feedback_" + question.index).remove();

    // if we don't have any feedback at all, return
    if (question.md.feedback == null && question.md.feedback_correct == null && question.md.feedback_incorrect == null && question.md.feedback_hint == null) {
      return;
    }
    var hint_fdb = $(question.jq).find('.query_mc_feedback_correct_incorrect');

    var hint_fdb_text = $(hint_fdb).text();
    $(hint_fdb).html('');

    var html = "<div class='question_feedback' id='question_feedback_" + question.index + "'>"
      + "<div class='question_feedback_head'>" + this.feedbackTitle + "</div>";

    // include general feedback if there
    if (question.md.feedback != null) {
      html += "<div class='question_feedback_part'>" + question.md.feedback + "</div>";
    }

    // include correct feedback if there and the answer is correct
    if (is_correct && question.md.feedback_correct != null) {
      html += "<div class='question_feedback_part'>" + question.md.feedback_correct + "</div>";
    }

    // include incorrect or hint feedback if there and the answer is incorrect
    if (!is_correct) {
      var found_feedback_string = false;
      // if user gets to try again...
      if (try_again) {
        // check for feedback_conditional's
        if (question.md.feedback_conditional.length > 0) {
          for (var i in question.md.feedback_conditional) {
            var fbc = question.check_conditional(question.md.feedback_conditional[i]);
            if (fbc != "") {
              html += "<div class='question_feedback_part'>" + fbc + "</div>";
              found_feedback_string = true;
              break;
            }
          }
        }

        // if we didn't find a feedback string to use above, check for feedback_hint
        if (!found_feedback_string && question.md.feedback_hint != null) {
          html += "<div class='question_feedback_part'>" + question.md.feedback_hint + "</div>";
          found_feedback_string = true;
        }
      }

      // if we didn't find a feedback string to use above, check for feedback_incorrect
      /*
      if (!found_feedback_string && question.md.feedback_incorrect != null) {
        html += "<div class='question_feedback_part'>" + question.md.feedback_incorrect + "<br/>" + hint_fdb_text + "</div>";
      }
      */
      if (!found_feedback_string && question.md.feedback_incorrect != null) {
        var incorrect_fb = question.md.feedback_incorrect.replace(/Incorrect\.\s+(.*)/, "Incorrect. <span class='visually-hidden'>$1</span>");

        html += "<div class='question_feedback_part'>" + incorrect_fb + "</div>";
      }
    }

    // close question_feedback div
    html += "</div>";

    // append the div, then fade it in
    $("#question_action_div_" + question.index).prepend(html);
  }
});


window.addEventListener('resize', function (event) {
  //console.log('caught resize event');
  resizeIframe();
});



// add a hidden div to detect changes in text size
//$('body').prepend('<div id="mutationWatcher" aria-hidden="true" class="visually-hidden"><span>x</span></div>');
/*
var mutationNode = document.getElementsByTagName('h1');
var mutationConfig = { attributes: true, childList: true, subtree: true };
var mutationCallback = function (mutationsList, observer) {
  console.log('mutation observer');
  console.log(mutationsList);
};
var mutationObserver = new MutationObserver(mutationCallback);
console.log('mutation observer created');
console.log(mutationObserver);  
mutationObserver.observe(mutationNode, mutationConfig);
*/


var Player_datavis = Player_manuscript_type.extend({

  hasAtLeastOneCBChecked: function (q_idx) {
    var question = player.activity.questions[q_idx];
    var cb_answered = false;

    question.queries.forEach(function (query) {
      //console.log('checking query: ', query);
      if (query.isChecked()) {
        cb_answered = true;
      }
    });

    return cb_answered;
  },

  calculateFigureIframeHeight: function () {

    //console.log('calculateFigureIframeHeight');

    var $iframeDiv = $('[data-figure_iframe="true"]');
    if ($iframeDiv.length < 1) {
      return 0;
    }
    var $iframe = $iframeDiv.children('iframe');
    if ($iframe.length < 1) {
      return 0;
    }

    //console.log($iframe);
    //console.log($iframe.contents().height());
    return $iframe.contents().find('html').height();
  },

  initialize: function () {

    var title = $('h1.section-title');
    var newTitle = title.find('[data-type=title]');
    this.activity_title = "<h1 id='h1_header' tabindex='-1'>" + newTitle.text() + "</h1>";
    title.remove();

    this._super();

    this.activity = new Activity_datavis();

    window.player.shadow_speech = new ShadowSpeech();
    $('.ada-shadow-speech').attr('tabIndex', '-1');
    if (platform.iOS || platform.mac) {
      $('#manuscript').prepend($('.ada-shadow-speech'));
    }
  },

  initialize2: function () {
    this._super();

    $('#digfir_section_0').attr('role', 'main');

    $('[data-type="question"] > h3').each(function () {
      var $this = $(this);
      $this.changeElementType('h2');
    });

    /*
    var mutationNode = document.getElementById('h1_header');
    var mutationConfig = { attributes: true, childList: true, subtree: true };
    var mutationCallback = function (mutationsList, observer) {
      console.log('mutation observer');
      console.log(mutationsList);
    };
    var mutationObserver = new MutationObserver(mutationCallback);
    console.log('mutation observer created');
    console.log(mutationObserver);
    mutationObserver.observe(mutationNode, mutationConfig);
    */
    
    $('[data-block_type="set_focus"]').each(function () {
      $(this).attr('tabindex', '-1');
    });


    $('td[align]').each(function () {
      var $this = $(this);
      var align = $this.attr('align');
      if (/^query_answer_/.test(align)) {
        var html = $(this).html();
        $this.html('<label for="' + align + '">' + html + '</label>');
      }
    });
    
  },

  show_completion_message: function () {

    // player.activity.grade_activity will always have been called before this fn
    // so we'll have access to the properly-calculated activity properties below
    var label = 'You have completed this activity. ' + 'Total Score: ' + player.activity.total_points_earned
      + ' out of ' + player.activity.total_points_possible
      + ' points (' + player.activity.grade_percent + '%).' + ' Your grade has been submitted to your instructor.';

    Standard_Dialog.alert(label, {
      'title': 'Congratulations!',
      'buttons': 'OK',
      'from': $('h1')
    });

    var macOSCheck = function () {
      return (navigator.userAgent.search(/Macintosh/i) !== -1);
    };
    //Dialog window have not role "dialog" on MacOS because of bug. VoiceOver stops reading of previous text when window with dialog role arise.
    //Therefore we update aria-live element here.
    if (macOSCheck()) {
      var iframeDiv = $('div[data-type="figure"] iframe');
      var $iframeBody = $(iframeDiv[0].contentWindow.document.body);
      var descNode = $iframeBody.find('#activeDescription');
      var firstPart = '';
      var secondPart = '';
      var questionFeedback = $('#question_feedback_10');
      var questionAction = $('#question_action_div_10');

      if (questionFeedback[0].childNodes[1]) {
        firstPart = questionFeedback[0].childNodes[1].innerHTML.replace(/<br>/g, '')
      }
      if (questionAction[0].childNodes[1]) {
        secondPart = questionAction[0].childNodes[1].innerHTML;
      }

      var desc = firstPart + ' ' + secondPart;
      setTimeout(function () {
        descNode.html(desc + '. ' + 'Congratulations!' + label + 'Web dialog. OK button');
      }, 100);

    }
  },
  show_navigation: function () {
    $("[data-block_type=activity_report]").hide();
    this._super();

    $("div[data-type='chapter']").prepend('<div class="section_title">' + player.activity_title + '<button role="button" aria-label="Go to the previous page" class="button_back">BACK</button></div>');
    $("div[data-block_type='description']").append('<button role="button" aria-label="Go to the next page" class="button_next">NEXT</button>');
    $(".button_back").hide();
    var bttnNext = document.querySelector('.button_next');
    var bttnBack = document.querySelector('.button_back');

    if (bttnNext) {
      bttnNext.addEventListener("click", function () {

        //fix focusable svg issue for IE
        var svgLoaded = setInterval(function () {
          var iframeDiv = $('div[data-type="figure"] iframe');
          var $svgElement = $(iframeDiv[0].contentWindow.document.body).find('svg');
          if ($svgElement.length) {
            clearInterval(svgLoaded);
            $svgElement.attr({
              'focusable': false,
              'aria-hidden': true
            });
          }
        }, 100);

        $("div[data-block_type='description']").hide();
        $("div[data-block_type='questions']").show();
        $(bttnBack).show();
        $('h1').focus();

        customizeVisibleQuestionRadioButtons();
        alignVisibleChoicesCells();
        g2dContainer = $($('div[data-type="figure"] iframe')[0].contentWindow.document.body).find('.commonG2D-container');
        iframeContainer = $('[data-block_type="visualization_activity"]');

        setTimeout(function () {
          currentG2DContainerHeight = g2dContainer.height();
          currentIfameContanerHeight = iframeContainer.height();


        }, 0);
        $(window).resize();

        setTimeout(function () {
          handleTextResize();
        },250);

      });
    }

    if (bttnBack) {
      bttnBack.addEventListener("click", function () {
        $("div[data-block_type='description']").show();
        $("div[data-block_type='questions']").hide();
        $(bttnBack).hide();
        $('h1').focus();
      });
    }

  }

});

player = new Player_datavis();

var CB_Query = CB_Query.extend({

  // return true if the CB is checked
  isChecked: function () {
    var input = document.getElementById('query_answer_'+this.query_index);
    return input.checked;
  },

  // CB questions are always considered answered because leaving them 
  // unchecked is a legitmate answer.
  isAnswered: function () {
    return true;
  }

});

var MC_Query = MC_Query.extend({

  isAnswered: function () {

    var isAnswered = false;
    var $question = $('[data-type="question"][data-question_index="' + this.parent_question.index + '"]');
    $question.find('input').each(function () {
      if (this.checked) {
        isAnswered = true;
      }
    });

    return isAnswered;
  },

  setError: function (errMesg) {
    //console.log('MC_Query.setError: ', this);
    $('#query_error_' + this.parent_question.index).remove();
    if (typeof errMesg === 'string') {
      $('#question_action_div_' + this.parent_question.index).prepend('<div id="query_error_' + this.parent_question.index + '" class="query_error_message">' + errMesg + '</div>');
    }
  },

  query_text_html: function () {

    //console.log('MC_Query.query_text_html: ', this);

    if (this.query_text != "" && this.md._hide_query_text != "true") {
      // replace newlines with <br />
      var qt = jQuery.trim(this.query_text).replace(/\n/g, "<br>")

      return "<legend class='query_text'>" + "<span class='visually-hidden'>Question " + this.parent_question.index + ", </span>" + qt + "</legend>"
    } else {
      return "";
    }
  },

  getHTML: function (mode) {
    var label_tag = 'label';
    // initialize imagemap elements
    if (this.imagemap_initialized === false) {
      var question_jq = this.jq.parent();
      this.im_init(question_jq);
      this.imagemap_initialized = true;
    }

    // start with the query text, if there
    var html = '<fieldset>';
    html += this.query_text_html();

    for (var z = 0; z < this.choices.length; z++) {
      var index = this.choice_order[z];

      // if this is review mode...
      var checked = '';
      var disabled = '';
      var ariaDisabled = '';
      var radio_style = 'query_mc_other_choice';

      if (mode == 'review' || mode == 'review_correct_incorrect' || mode == 'preview') {
        // if it's the correct choice, it should be marked in green
        if (index == this.correct_answer_index) {
          // but only if we're *not* in review_correct_incorrect mode or this is the user's choice
          if (mode != 'review_correct_incorrect' || index == this.user_answer_index) {
            radio_style = 'query_mc_correct_choice';
          }

          // if this is what the user chose, he got it correct
          if (index == this.user_answer_index) {
            is_correct = true;
          }

          // else if it's the user's choice (and this.md.no_correct_answer is not true), it should be marked in red
        }
        else if (index == this.user_answer_index && this.md.no_correct_answer != 'true') {
          radio_style = 'query_mc_incorrect_user_choice';
        }

        // and the input should be disabled, unless we're in review_correct_incorrect mode
        if (mode != 'review_correct_incorrect') {
          disabled = platform.android ? '' : 'disabled';
          ariaDisabled = platform.android ? 'aria-disabled="true"' : '';
        }
      }

      // if this is the choice the user clicked, it's checked (regardless of mode)
      if (index == this.user_answer_index) {
        checked = 'checked';
      }

      var mc_option_class = '';
      var mc_answer_info = '';

      
      //var ariaHidden = platform.ios ? 'aria-hidden="true" role="presentation"' : '';
      var answerInfoID = 'answer-fb-info-' + this.query_index + '_' + index;
      //var labelId = 'aria-labelledby="query_answer_label_' + this.query_index + '_' + index + '"';

      /*
      if (platform.android) {
        labelId = 'aria-labelledby="query_answer_label_' + this.query_index + '_' + index + '"';
      }

      if (platform.ios || platform.ie) {
        labelId = 'aria-labelledby="query_answer_label_' + this.query_index + '_' + index + ' ' + answerInfoID + '"';
      }
      */

      if (radio_style === 'query_mc_correct_choice') {
        mc_option_class = ' correct_answer';

        if (checked === 'checked') {
          mc_answer_info = ' <span id="' + answerInfoID + '" class="answer_info correct visually-hidden" '
            + '>, [Your answer, correct]</span>';
        }
        else {
          mc_answer_info = ' <span  id="' + answerInfoID + '" class="answer_info correct visually-hidden" '
            + '>, [Correct answer]</span>';
        }
      }
      else if (radio_style === 'query_mc_incorrect_user_choice') {
        mc_option_class = ' incorrect_answer';

        if (checked === 'checked') {
          mc_answer_info = ' <span  id="' + answerInfoID + '" class="answer_info incorrect visually-hidden" '
            + '>, [Your answer, incorrect]</span>';
        }
        else {
          mc_answer_info = ' <span  id="' + answerInfoID + '" class="answer_info incorrect visually-hidden" '
            + ariaHidden + '>, [Incorrect answer]</span>';
        }
      }


      // Create radio button input plus label
      //html += '<div class="mc_option' + mc_option_class + '"><span class="' + radio_style
      html += '<div class="mc_option' + mc_option_class + '">'
        //+ '"><input class="query_mc_choice_radio" type="radio" name="query_answer_' + this.query_index + '" '
        + '<input class="query_mc_choice_radio" type="radio" name="query_answer_' + this.query_index + '" '
        + ' id="query_answer_' + this.query_index + '_' + index + '" '
        + ' value="' + index + '" '
        + checked + ' '
        + disabled + ' '
        + ariaDisabled + ' '
        + ' ' + '/>'
        //+ '</span>'
        + '<' + label_tag + ' id="query_answer_label_' + this.query_index + '_' + index + '"'
        + 'class="text"';

      if (label_tag === 'label') {
        html += ' for="query_answer_' + this.query_index + '_' + index + '"';
      }

      var answer_info = platform.ios
        ? {
          inside: '',
          outside: mc_answer_info
        }
        : {
          inside: mc_answer_info,
          outside: ''
        };

      html += '>'
        + this.choices[index]
        + answer_info.inside
        + '</' + label_tag + '></div>' + answer_info.outside;
    }

    html += '</fieldset>';

    // some useful fixes
    if (html.match(/__/)) {
      html = html.replace(/(<span data-block_type="underline">)_+(<\/span>)/g, '<span class="v-h">blank</span>$1$2');
    }

    if (platform.mac) {
      var firstLabelId = html.substr(html.search('label id="')).split('"')[1];

      html = html
        .replace('legend', 'div')
        .replace(/(query_text)/, '$1" id="query_text_' + this.parent_question.index + '"')
        .replace(/(type="radio")/, '$1 aria-labelledby="query_text_'
          + this.parent_question.index + ' ' + firstLabelId + '"')
        .replace('/legend', '/div');
    }

    return html;
  }

});


function ShadowSpeech(aria_live, role, atomic, root) {
  var frag;

  root = root || document.body;
  this.live = aria_live || (platform.IE ? 'assertive' : 'polite');
  this.role = role ? 'role = "' + role + '"' : '';
  this.atomic = atomic ? 'aria-atomic = "' + atomic + '"' : '';

  frag = $('<div class="ada-shadow-speech"' + this.role + 'aria-live="' + this.live + '"' + this.atomic + '></div>');

  $(root).append(frag);

  this._el = $(frag);
}

ShadowSpeech.prototype.talk = function (some, clone) {
  if (clone && some instanceof Node) {
    some = some.cloneNode(true);
  }

  this._el.html('');
  this._el.html(some);
  // dom.clear(this._el);
  // dom.setHtml(some, this._el);
};
